/*
	RC4 Documents - Hash functions for old Word, Excel, PDF documents.
	Copyright (C) 2013 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef COMMON_H
#define COMMON_H

#include "architecture.h"

#ifdef _WIN32
	#pragma warning(disable:4996)
#endif

#if defined(_MSC_VER) || defined(__INTEL_COMPILER)
	#define __inline__  __forceinline
#elif defined(__GNUC__)
	#define __inline__  __inline __attribute__((__always_inline__))
#else
	#define __inline__  inline
#endif

#define __STDC_CONSTANT_MACROS
#define __STDC_FORMAT_MACROS
#define __STDC_LIMIT_MACROS
#ifdef _WIN32
	#include "inttypes.h"
	#include <windows.h>
	typedef LARGE_INTEGER TIMER_TYPE;
	#define TIMER_FUNC(t)  QueryPerformanceCounter(&t)
	__inline__ double TIMER_DIFF(LARGE_INTEGER s, LARGE_INTEGER e)
	{
		LARGE_INTEGER f;
		QueryPerformanceFrequency(&f);
		return ((double) (e.QuadPart - s.QuadPart)) / f.QuadPart;
	}
#else
	#include <inttypes.h>
	#include <sys/time.h>
	typedef timeval TIMER_TYPE;
	#define TIMER_FUNC(t)    gettimeofday(&t, NULL)
	#define TIMER_DIFF(s,e)  ((e.tv_sec - s.tv_sec) + (e.tv_usec - s.tv_usec) / (double)1000000.0)
#endif

#define SWAP_ENDIAN_32(x)           (((x) >> 24) | ((x) << 24) | (((x) >> 8) & 0xff00) | (((x) << 8) & 0xff0000))

#ifdef ARC_LITTLE_ENDIAN
	#define TO_BIG_ENDIAN_32(x)     SWAP_ENDIAN_32(x)
	#define TO_LITTLE_ENDIAN_32(x)  (x)
#else
	#define TO_BIG_ENDIAN_32(x)     (x)
	#define TO_LITTLE_ENDIAN_32(x)  SWAP_ENDIAN_32(x)
#endif
#define FROM_BIG_ENDIAN_32(x)       TO_BIG_ENDIAN_32(x)
#define FROM_LITTLE_ENDIAN_32(x)    TO_LITTLE_ENDIAN_32(x)

#endif
